@@ -3,10 +3,11 @@
 #include <array>
 #include <vector>
 
-#include <tests/reference/validate_utf16.h>
-#include <tests/helpers/transcode_test_base.h>
+#include <tests/helpers/fixed_string.h>
 #include <tests/helpers/random_int.h>
 #include <tests/helpers/test.h>
+#include <tests/helpers/transcode_test_base.h>
+#include <tests/reference/validate_utf16.h>
 
 namespace {
 constexpr std::array<size_t, 7> input_size{7, 16, 12, 64, 67, 128, 256};
@@ -165,4 +166,48 @@ TEST(all_possible_8_codepoint_combinations) {
   }
 }
 
+#if SIMDUTF_CPLUSPLUS23
+
+namespace {
+template <auto input> constexpr auto size() {
+  return simdutf::utf32_length_from_utf16(input);
+}
+template <auto input> constexpr auto convert() {
+  using namespace simdutf::tests::helpers;
+  CTString<char32_t, size<input>()> tmp;
+  const auto ret = simdutf::convert_utf16_to_utf32(input, tmp);
+  if (ret != tmp.size()) {
+    throw "unexpected write size";
+  }
+  return tmp;
+}
+} // namespace
+
+TEST(compile_time_convert_utf16_to_utf32) {
+  using namespace simdutf::tests::helpers;
+  static_assert(convert<u"köttbulle"_utf16>() == U"köttbulle"_utf32);
+}
+
+namespace {
+template <auto input> constexpr auto size_le() {
+  return simdutf::utf32_length_from_utf16le(input);
+}
+template <auto input> constexpr auto convert_le() {
+  using namespace simdutf::tests::helpers;
+  CTString<char32_t, size_le<input>()> tmp;
+  const auto ret = simdutf::convert_utf16le_to_utf32(input, tmp);
+  if (ret != tmp.size()) {
+    throw "unexpected write size";
+  }
+  return tmp;
+}
+} // namespace
+
+TEST(compile_time_convert_utf16le_to_utf32) {
+  using namespace simdutf::tests::helpers;
+  static_assert(convert_le<u"köttbulle"_utf16le>() == U"köttbulle"_utf32);
+}
+
+#endif
+
 TEST_MAIN
