@@ -1,11 +1,11 @@
 #include "simdutf.h"
 
 #include <array>
-#include <vector>
 
-#include <tests/helpers/transcode_test_base.h>
+#include <tests/helpers/fixed_string.h>
 #include <tests/helpers/random_int.h>
 #include <tests/helpers/test.h>
+#include <tests/helpers/transcode_test_base.h>
 
 namespace {
 constexpr std::array<size_t, 7> input_size{7, 16, 12, 64, 67, 128, 256};
@@ -59,4 +59,34 @@ TEST_LOOP(convert_into_2_or_4_UTF16_bytes) {
   }
 }
 
+#if SIMDUTF_CPLUSPLUS23
+
+namespace {
+template <auto input> constexpr auto size() {
+  return simdutf::utf16_length_from_utf32(input);
+}
+
+template <auto input> constexpr auto convert_be() {
+  using namespace simdutf::tests::helpers;
+  CTString<char16_t, size<input>(), std::endian::big> tmp;
+  const auto ret = simdutf::convert_valid_utf32_to_utf16be(input, tmp);
+  if (ret != tmp.size()) {
+    throw "unexpected write size";
+  }
+  return tmp;
+}
+} // namespace
+
+TEST(compile_time_convert_valid_utf32_to_utf16be) {
+  using namespace simdutf::tests::helpers;
+
+  constexpr auto input = U"köttbulle"_utf32;
+  constexpr auto expected = u"köttbulle"_utf16be;
+  constexpr bool with_errors = true;
+  constexpr auto output = convert_be<input>();
+  static_assert(output == expected);
+}
+
+#endif
+
 TEST_MAIN
